package com.vpn.fastestvpnservice.screens.accountScreensAll

import android.content.Context
import android.net.VpnService
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.activity.ComponentActivity
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.LocalOverscrollConfiguration
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.BoxScope
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.itemsIndexed
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.Icon
import androidx.compose.material.IconButton
import androidx.compose.material.Surface
import androidx.compose.material.Text
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.CircularProgressIndicator
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.CompositionLocalProvider
import androidx.compose.runtime.DisposableEffect
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.runtime.mutableFloatStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberUpdatedState
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.ComposeView
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalLifecycleOwner
import androidx.compose.ui.res.colorResource
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.FontStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.core.app.ActivityCompat
import androidx.fragment.app.Fragment
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleEventObserver
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.NavHostController
import androidx.navigation.compose.rememberNavController
import com.android.billingclient.api.BillingClient
import com.android.billingclient.api.BillingClientStateListener
import com.android.billingclient.api.BillingResult
import com.android.billingclient.api.PurchasesUpdatedListener
import com.android.billingclient.api.SkuDetails
import com.vpn.fastestvpnservice.R
import com.vpn.fastestvpnservice.beans.DataResponse
import com.vpn.fastestvpnservice.beans.ProductFeatures
import com.vpn.fastestvpnservice.beans.Server
import com.vpn.fastestvpnservice.beans.subscriptionList
import com.vpn.fastestvpnservice.beans.subscriptionPackageList
import com.vpn.fastestvpnservice.customItems.SubscriptionDataItem
import com.vpn.fastestvpnservice.customItems.SubscriptionPackageItem
import com.vpn.fastestvpnservice.customItems.getSelectedPosition
import com.vpn.fastestvpnservice.helpers.BasePreferenceHelper
import com.vpn.fastestvpnservice.screens.bottomNavBarScreens.OnLifecycleEvent
import com.vpn.fastestvpnservice.sealedClass.Screen
import com.vpn.fastestvpnservice.ui.theme.FastestVPNTheme
import com.vpn.fastestvpnservice.ui.theme.customTypography2
import com.vpn.fastestvpnservice.utils.Utils
import com.vpn.fastestvpnservice.viewmodels.BillingViewModel
import com.vpn.fastestvpnservice.viewmodels.SubscriptionViewModel
import kotlinx.coroutines.delay
import kotlin.random.Random
import kotlin.random.nextInt


@OptIn(ExperimentalFoundationApi::class)
@Composable
fun SubscriptionScreen(navHostController: NavHostController, activity: ComponentActivity)
{

    val context = LocalContext.current
    val subscriptionViewModel: SubscriptionViewModel = viewModel{
        SubscriptionViewModel(context = context, activity)
    }
    var selectedPosition: Int = 0
    var skuDetailsList: List<SkuDetails>? = null
    val basePreferenceHelper = BasePreferenceHelper(context)
    var isLoaderShow by remember { mutableStateOf(true) }

    Box(
        modifier = Modifier
            .fillMaxSize()
            .background(MaterialTheme.colorScheme.background)
    ) {

        LaunchedEffect(key1 = true) {
            Log.d("test_api_response_p", "Products API called!")
            subscriptionViewModel.getProducts(onProductResult = {
                isLoaderShow = false
            })
        }

        if (isLoaderShow) {
            var progress by remember { mutableFloatStateOf(0.1F) }

            LaunchedEffect(key1 = Unit) {
                while (true) {
                    for (i in 1..100) {
                        progress = i.toFloat()/100F
                        delay(150)
                    }
                    progress = 0.1F
                }
            }

            CircularProgressIndicator(
                progress = { progress },
                modifier = Modifier
                    .size(50.dp)
                    .align(Alignment.Center),
                color = colorResource(id = R.color.yellow_text),
                strokeWidth = 5.dp,
            )
        }

        HeaderRowSS(navHostController = navHostController)

        Column(
            verticalArrangement = Arrangement.Top,
            horizontalAlignment = Alignment.Start,
            modifier = Modifier
                .padding(
                    top = 120.dp,
                    start = 16.dp, end = 16.dp,
                    bottom = 120.dp
                )
                .fillMaxSize()
                .background(Color.Transparent)
        ) {
            Text(text = "Upgrade Your Account",
                color = MaterialTheme.colorScheme.primary,
                style = MaterialTheme.typography.titleMedium
            )

            Column(
                verticalArrangement = Arrangement.Top,
                horizontalAlignment = Alignment.Start,
                modifier = Modifier.padding(top = 30.dp)
            ) {
                CompositionLocalProvider(
                    LocalOverscrollConfiguration provides null
                ) {
                    val selectedPosition1 = SubscriptionViewModel.selectedPosition
                    val featuresList = subscriptionViewModel.featuresListSortedItems.observeAsState().value
                    featuresList?.let {
                        Log.d("test_subscription_list", "featuresListSorted[SS] size = ${it.size}")

                    }
                    val selectPriceList = subscriptionViewModel.selectPriceList.observeAsState().value

                    LazyColumn {

                        featuresList?.let {

                            it.let {
                                Log.d("test_api_response_p",
                                    "Products screen = ${it.size}, selectedPosition = $selectedPosition1")

                                if (it.size > 0) {
                                    isLoaderShow = false
//                                    val features = selectedPosition1.let { it1 -> it.features }
                                    it.let {
                                        items(items = it) {item ->
                                            SubscriptionDataItem(item = item)
                                        }
                                    }

                                    item { Spacer(modifier = Modifier.height(20.dp)) }

                                    selectPriceList?.let {
                                        itemsIndexed(items = it) { position, plan ->
                                            Log.d("test_plan_product", "SS[$position] -> ${plan.packageDuration} ${plan.isSelected.value}")
                                            SubscriptionPackageItem(item = plan, position, activity)
                                        }
                                    }

                                    item { Spacer(modifier = Modifier.height(20.dp)) }
                                }
                            }
                        }
                    }
                }
            }
        }

        val skuDetailsListsSubs = subscriptionViewModel.liveDataSkuDetails.observeAsState().value

        Button(

            onClick = {
                selectedPosition = getSelectedPosition()
//                Toast.makeText(context, "$selectedPosition", Toast.LENGTH_SHORT).show()
                Log.d("test_button", "onClick")
//                subscriptionViewModel.mutableLiveDataNumber.value = 1
                Log.d("SubscriptionViewModel", "liveData ${skuDetailsListsSubs}")

//                subscriptionViewModel.startConnection(selectedPosition)
                if (skuDetailsListsSubs != null) {
                    subscriptionViewModel.subscriptionProduct(
                        selectedPosition, skuDetailsListsSubs, activity
                    )
                }

            },
            modifier = Modifier
                .padding(
                    top = 29.dp,
                    start = 16.dp, end = 16.dp,
                    bottom = 34.dp
                )
                .align(Alignment.BottomCenter)
                .background(colorResource(id = R.color.transparent))
                .fillMaxWidth()
                .height(60.dp),
            shape = RoundedCornerShape(15.dp),
//            border = BorderStroke(25.dp, colorResource(id = R.color.black)),
            colors = ButtonDefaults.buttonColors(
                contentColor = colorResource(id = R.color.white),
                containerColor = colorResource(id = R.color.blue_text),
            ),
        )
        {
            Text(text = "Subscribe",
                color = colorResource(id = R.color.white),
                style = MaterialTheme.typography.customTypography2.bodySmall
            )
        }
    }
}


@Composable
fun OnLifecycleEventSS(onEvent: (
    owner: LifecycleOwner,
    event: Lifecycle.Event
) -> Unit) {
    val eventHandler = rememberUpdatedState(onEvent)
    val lifecycleOwner = rememberUpdatedState(LocalLifecycleOwner.current)

    DisposableEffect(lifecycleOwner.value) {
        val lifecycle = lifecycleOwner.value.lifecycle
        val observer = LifecycleEventObserver { owner, event ->
            eventHandler.value(owner, event)
        }

        lifecycle.addObserver(observer)
        onDispose {
            lifecycle.removeObserver(observer)
        }
    }
}

@Composable
fun BoxScope.HeaderRowSS(navHostController: NavHostController) {
    IconButton(
        onClick = {
            navHostController.popBackStack()
//                    navHostController.navigate(BottomBarScreen.Help.route)
        },
        modifier = Modifier
            .align(Alignment.TopStart)
            .padding(top = 50.dp)
            .padding(start = 16.dp)
            .size(30.dp, 32.dp)
    ) {
        Icon(
            painter = painterResource(id = R.drawable.backarrow3x),
            contentDescription = "Arrow-Back",
            tint = MaterialTheme.colorScheme.primary,
            modifier = Modifier.size(18.dp, 12.dp)
        )
    }
    Surface(
        modifier = Modifier
            .padding(top = 50.dp)
            .height(32.dp)
            .align(Alignment.TopCenter)
            .padding(5.dp),
        color = colorResource(id = R.color.transparent)
    ) {
        Text(text = "Subscription",
            color = MaterialTheme.colorScheme.primary,
            style = MaterialTheme.typography.bodyMedium,
            modifier = Modifier.fillMaxHeight()
        )
    }
}

@Preview
@Composable
fun SubscriptionScreenPreview() {
//    SubscriptionScreen(rememberNavController(), ComponentActivity())
}